// PeriodMeasure.c
// Runs on TM4C1294
// Use Timer0A in 24-bit edge time mode to request interrupts on the rising
// edge of PL4 (T0CCP0), and measure period between pulses.
// Daniel Valvano and Jonathan Valvano
// September 30, 2014
/*
 This example accompanies the book
  "Embedded Systems: Introduction to ARM Cortex M Microcontrollers",
  ISBN: 978-1469998749, Jonathan Valvano, copyright (c) 2014
  Program 4.4

 Copyright 2014 by Jonathan W. Valvano, valvano@mail.utexas.edu
    You may use, edit, run or distribute this file
    as long as the above copyright notice remains
 THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 For more information about my classes, my research, and my books, see
 http://users.ece.utexas.edu/~valvano/
 */

#include <stdint.h>
#include "..//tm4c1294ncpdt.h"
#include "PLL.h"

#define PN0                     (*((volatile uint32_t *)0x40064004))
#define PN1                     (*((volatile uint32_t *)0x40064008))

#define SYSCTL_RCGCGPIO_R       (*((volatile uint32_t *)0x400FE608))
#define SYSCTL_RCGCGPIO_R12     0x00001000  // GPIO Port N Run Mode Clock Gating Control, bit 12
#define SYSCTL_RCGCGPIO_R10     0x00000400  // GPIO Port L Run Mode Clock Gating Control, bit 10
#define SYSCTL_PRGPIO_R         (*((volatile uint32_t *)0x400FEA08))
#define SYSCTL_PRGPIO_R12       0x00001000  // GPIO Port N Peripheral Ready
#define SYSCTL_PRGPIO_R10       0x00000400  // GPIO Port L Peripheral Ready
#define TIMER_TAMR_TACMR        0x00000004  // GPTM TimerA Capture Mode
#define TIMER_TAMR_TAMR_CAP     0x00000003  // Capture mode
#define TIMER_CTL_TAEN          0x00000001  // GPTM TimerA Enable
#define TIMER_CTL_TAEVENT_POS   0x00000000  // Positive edge
#define TIMER_CTL_TAEVENT_NEG   0x00000004  // Negative edge
#define TIMER_CTL_TAEVENT_BOTH  0x0000000C  // Both edges
#define TIMER_IMR_CAEIM         0x00000004  // GPTM CaptureA Event Interrupt
                                            // Mask
#define TIMER_ICR_CAECINT       0x00000004  // GPTM CaptureA Event Interrupt
                                            // Clear
#define TIMER_TAILR_TAILRL_M    0x0000FFFF  // GPTM TimerA Interval Load
                                            // Register Low
void DisableInterrupts(void); // Disable interrupts
void EnableInterrupts(void);  // Enable interrupts
long StartCritical (void);    // previous I bit, disable interrupts
void EndCritical(long sr);    // restore I bit to previous value
void WaitForInterrupt(void);  // low power mode

uint32_t Period;              // (1/clock) units
uint32_t First;               // Timer0A first edge
int32_t Done;                // set each rising
// max period is (2^24-1)*8.33ns = 139.8ms
// min period determined by time to run ISR, which is about 1us
void PeriodMeasure_Init(void){
  SYSCTL_RCGCTIMER_R |= 0x01;// activate timer0  
  SYSCTL_RCGCGPIO_R |= 0x00001400; // activate clock for Port N, L
  while((SYSCTL_PRGPIO_R&0x00001400) != 0x00001400){};
  First = 0;                       // first will be wrong
  Done = 0;                        // set on subsequent
  GPIO_PORTN_DIR_R |= 0x03;        // make PN1-0 out (PN1-0 built-in LED1-2)
  GPIO_PORTN_AFSEL_R &= ~0x03;     // disable alt funct on PN1-0
  GPIO_PORTN_DEN_R |= 0x03;        // enable digital I/O on PN1-0
                                   // configure PN1-0 as GPIO
  GPIO_PORTN_PCTL_R = (GPIO_PORTN_PCTL_R&0xFFFFFF00)+0x00000000;
  GPIO_PORTN_AMSEL_R &= ~0x03;     // disable analog functionality on PN1-0

  GPIO_PORTL_DIR_R &= ~0x10;       // make PL4 (T0CCP0) in
  GPIO_PORTL_AFSEL_R |= 0x10;      // enable alt funct on PL4 (T0CCP0)
  GPIO_PORTL_DEN_R |= 0x10;        // enable digital I/O on PL4 (T0CCP0)
                                   // configure PL4 as T0CCP0
  GPIO_PORTL_PCTL_R = (GPIO_PORTL_PCTL_R&0xFFF0FFFF)+0x00030000;
  GPIO_PORTL_AMSEL_R &= ~0x10;     // disable analog functionality on PL4 (T0CCP0)

  TIMER0_CTL_R &= ~TIMER_CTL_TAEN; // disable timer0A during setup
  TIMER0_CFG_R = TIMER_CFG_16_BIT; // configure for 16-bit timer mode
                                   // configure for 24-bit capture mode
  TIMER0_TAMR_R = (TIMER_TAMR_TACMR|TIMER_TAMR_TAMR_CAP);
                                   // configure for rising edge event
  TIMER0_CTL_R &= ~(TIMER_CTL_TAEVENT_POS|0xC);
  TIMER0_TAILR_R = 0x0000FFFF;     // start value
  TIMER0_TAPR_R = 0xFF;            // activate prescale, creating 24-bit
  TIMER0_IMR_R |= TIMER_IMR_CAEIM; // enable capture match interrupt
  TIMER0_ICR_R = TIMER_ICR_CAECINT;// clear timer0A capture match flag
  TIMER0_CTL_R |= TIMER_CTL_TAEN;  // enable timer0A 16-b, +edge timing, interrupts
                                   // Timer0A=priority 2
  NVIC_PRI4_R = (NVIC_PRI4_R&0x00FFFFFF)|0x40000000; // top 3 bits
  NVIC_EN0_R = 1<<19;              // enable interrupt 19 in NVIC
}
void Timer0A_Handler(void){
  PN1 = PN1^0x02;  // toggle PN1
  PN1 = PN1^0x02;  // toggle PN1
  TIMER0_ICR_R = TIMER_ICR_CAECINT;// acknowledge timer0A capture match
  Period = (First - TIMER0_TAR_R)&0xFFFFFF;// 24 bits, 8.33ns resolution
  First = TIMER0_TAR_R;            // setup for next
  Done = 1;
  PN1 = PN1^0x02;  // toggle PN1
}

//debug code
int main(void){           
  PLL_Init(); // 120 MHz

  PeriodMeasure_Init();            // initialize 24-bit timer0A in capture mode
  EnableInterrupts();
  while(1){
    WaitForInterrupt();
    PN0 = PN0^0x01;  // toggle PN0
  }
}
